@@ -7,6 +7,7 @@ import gc
 from graph_kmer_index import kmer_hash_to_sequence, sequence_to_kmer_hash
 from npstructures import Counter, HashTable
 from Bio.Seq import Seq
+from .flat_kmers import FlatKmers
 
 from .multi_value_hashtable import MultiValueHashTable
 
@@ -17,18 +18,26 @@ class CounterKmerIndex:
         self.counter = counter
 
     @classmethod
-    def from_kmer_index(cls, kmer_index, modulo):
+    def from_kmer_index(cls, kmer_index):
         kmers = kmer_index._kmers.astype(np.int64)
         nodes = kmer_index._nodes
         unique_kmers = np.unique(kmers)
-        counter = Counter(unique_kmers, np.zeros_like(unique_kmers), mod=modulo)
+        logging.info("N unique kmers: %d" % len(unique_kmers))
+        #counter = Counter(unique_kmers, np.zeros_like(unique_kmers, dtype=np.uint16), mod=modulo, value_dtype=np.uint16)
+        counter = Counter(unique_kmers, 0, mod=kmer_index._modulo, value_dtype=np.uint16)
         return cls(kmers, nodes, counter)
 
-    def count_kmers(self, kmers):
+    def reset(self):
+        self.counter = np.zeros_like(self.counter)
+
+    def count_kmers(self, kmers, update_counter=True):
+        if not update_counter:
+            self.reset()
+
         self.counter.count(kmers.astype(np.int64))
 
-    def get_node_counts(self):
-        return np.bincount(self.nodes, self.counter[self.kmers])
+    def get_node_counts(self, min_nodes=0):
+        return np.bincount(self.nodes, self.counter[self.kmers], minlength=min_nodes)
 
 
 
@@ -259,6 +268,11 @@ class CollisionFreeKmerIndex:
     def __contains__(self, item):
         return self.get(int(item), 100000000000)[0] is not None
 
+
+    def get_nodes(self, kmer, max_hits=10):
+        nodes, offsets, frequencies, allele_frequencies = self.get(kmer, max_hits)
+        return nodes
+
     def get(self, kmer, max_hits=10):
         hash = kmer % self._modulo
         position = self._hashes_to_index[int(hash)]
@@ -425,6 +439,26 @@ class CollisionFreeKmerIndex:
         return object
 
 
+    def convert_kmers_to_complement(self, k=31, skip_frequencies=True):
+        from .kmer_hashing import kmer_hashes_to_complement_hashes
+        new_kmers = []
+        for i, kmer_chunk in enumerate(np.array_split(self._kmers, len(self._kmers)//10000000)):
+            logging.info("Doing chunk %d" % i)
+            new_kmers.append(kmer_hashes_to_complement_hashes(kmer_chunk, k))
 
+        logging.info("Concatenating")
+        new_kmers = np.concatenate(new_kmers)
+
+        logging.info("Building")
+        return CollisionFreeKmerIndex.from_flat_kmers(
+            FlatKmers(
+                new_kmers,
+                self._nodes,
+                self._ref_offsets,
+                self._allele_frequencies,
+            ),
+            modulo=self._modulo,
+            skip_frequencies=skip_frequencies
+        )
 
 
