@@ -35,7 +35,6 @@ from .critical_graph_paths import CriticalGraphPaths
 from shared_memory_wrapper import to_file, from_file
 from obgraph.position_id import PositionId
 
-from kmer_mapper.util import log_memory_usage_now
 
 
 def main():
@@ -63,6 +62,7 @@ def create_index_single_thread(args, interval=None):
             fasta = Fasta(args.reference_fasta)
             logging.info("Names in fasta: %s" % str(fasta.keys()))
             reference = fasta[args.reference_name]
+            assert len(reference) > 0, "Length of ref sequennce is 0. Seomthing is wrong"
         except KeyError:
             logging.error("Did not find reference name %s in %s" % (args.reference_name, args.reference_fasta))
             raise
@@ -97,9 +97,7 @@ def create_index_single_thread(args, interval=None):
 
     if args.include_reverse_complement:
         logging.info("Adding reverse complements")
-        log_memory_usage_now("before adding reverse complements")
         kmers_revcomp = kmers.get_reverse_complement_flat_kmers(args.kmer_size)
-        log_memory_usage_now("after adding reverse complements")
         kmers = FlatKmers.from_multiple_flat_kmers([kmers, kmers_revcomp])
 
     return kmers
@@ -329,7 +327,8 @@ def run_argument_parser(args):
                                           node_to_variants=node_to_variants,
                                           do_not_choose_lowest_frequency_kmers=args["do_not_choose_lowest_frequency_kmers"],
                                           use_dense_kmer_finder=use_dense_kmer_finder,
-                                          position_id_index=args["position_id_index"]
+                                          position_id_index=args["position_id_index"],
+                                          use_simple=args["simple"]
                                           )
         flat_kmers = finder.find_unique_kmers()
         return flat_kmers
@@ -370,8 +369,33 @@ def run_argument_parser(args):
     subparser.add_argument("-c", "--chunk-size", required=False, default=10000, type=int, help="Number of variants given to each thread")
     subparser.add_argument("-m", "--max-variant-nodes", required=False, default=6, type=int, help="Maximum number of variant nodes allowed in kmer")
     subparser.add_argument("-d", "--do-not-choose-lowest-frequency-kmers", required=False, type=bool, help="For testing only. Will not choose the best kmers.")
+    subparser.add_argument("-S", "--simple", type=bool, default=False, help="Set to True to use simple kmer selection")
     subparser.set_defaults(func=make_unique_variant_kmers)
 
+
+    def sample_kmers_from_structural_variants_command(args):
+        from .structural_variants import sample_kmers_from_structural_variants
+        kmers = sample_kmers_from_structural_variants(args.graph,
+                                                      args.variant_to_nodes,
+                                                      args.kmer_index,
+                                                      args.kmer_size)
+        kmers.to_file(args.out_file_name)
+
+
+    subparser = subparsers.add_parser("sample_kmers_from_structural_variants", help="Samples extra kmers from large structural variant nodes. Meant ot be combined with other variant kmers")
+    subparser.add_argument("-g", "--graph", required=True, type=Graph.from_file)
+    subparser.add_argument("-V", "--variant_to_nodes", required=True, type=VariantToNodes.from_file)
+    subparser.add_argument("-k", "--kmer-size", required=True, type=int)
+    subparser.add_argument("-i", "--kmer-index", required=False,
+                           help="Kmer index used to check frequency of kmers in genome",
+                           type=from_file)
+    subparser.add_argument("-I", "--kmer-counter", required=False,
+                           help="Kmer index used to check frequency of kmers in genome", type=from_file)
+    subparser.add_argument("-o", "--out-file-name", required=True)
+    subparser.add_argument("-t", "--n-threads", required=False, default=1, type=int)
+    subparser.set_defaults(func=sample_kmers_from_structural_variants_command)
+
+
     subparser = subparsers.add_parser("prune_flat_kmers")
     subparser.add_argument("-f", "--flat-index", required=True)
     subparser.add_argument("-o", "--out-file-name", required=True)
@@ -445,7 +469,6 @@ def run_argument_parser(args):
         args, chunk = data
         args = object_from_shared_memory(args)
         logging.info("Processing chunk %s" % str(chunk))
-        log_memory_usage_now(str(chunk))
 
         t = time.perf_counter()
         kmer_finder = DenseKmerFinder(args["graph"], args["kmer_size"], critical_graph_paths=args["critical_graph_paths"],
@@ -474,9 +497,7 @@ def run_argument_parser(args):
 
         args = vars(args)
         args.pop("func")
-        log_memory_usage_now("Before shared memory")
         args_shared = object_to_shared_memory(args)
-        log_memory_usage_now("After shared memory")
         critical_paths = args["critical_graph_paths"]
 
         n_chunks = args["n_threads"]*20
@@ -501,7 +522,6 @@ def run_argument_parser(args):
         for i, results in enumerate(pool.imap(index_single_thread, zip(itertools.repeat(args_shared), chunks))):
             flat_kmers.append(results)
             logging.info("Done with chunk. Found %d kmers" % (len(results._hashes)))
-            log_memory_usage_now("Done with chunk %d" % i)
 
         logging.info("Time spent to make indexes: %.2f" % (time.perf_counter()-t))
         close_shared_pool()
