@@ -6,20 +6,30 @@ from .nplist import NpList
 import sys
 sys.setrecursionlimit(20000)
 from obgraph.position_id import PositionId
+from . import kmer_hash_to_sequence
+from .kmer_hashing import power_array, reverse_power_array
+
+_complement_lookup = np.array([3, 2, 1, 0], dtype=np.uint64)
+
 
 def update_hash(current_base, current_hash, first_base, k, only_add=False):
-    # only_add=True means to build hash without subtracting previous first base
+    # only_add!=True means to build hash without subtracting previous first base
+    # only_add should then be the number of the base we are at
     # very important that everything is int (not float) when working with large k
     current_hash = int(current_hash)
     #assert type(current_hash) == int, "Current hash has type %s" % type(current_hash)
     current_base = int(current_base)
     first_base = int(first_base)
-    current_base_complement = (current_base + 2) % 4
-    if only_add:
-        current_hash = current_hash * 4 + current_base
+    current_base_complement = _complement_lookup[current_base]  # (current_base + 2) % 4
+    #print("Current base: %d. Only add: %s" % (current_base, only_add))
+    if not isinstance(only_add, bool):
+        #current_hash = current_hash * 4 + current_base
+        current_hash = current_hash + 4**(only_add) * current_base
+        #print("ONLY ADDING HASH. Current hash: %d, only add: %d" % (current_hash, only_add))
     else:
-        current_hash = (current_hash - first_base * 4 ** (k - 1)) * 4 + current_base
-        first_base_complement = (first_base + 2) % 4
+        #current_hash = (current_hash - first_base * 4 ** (k - 1)) * 4 + current_base
+        current_hash = (current_hash - first_base) // 4 + current_base * 4**(k-1)
+        first_base_complement = _complement_lookup[first_base]  # (first_base + 2) % 4
 
     return current_hash
 
@@ -33,7 +43,8 @@ class DenseKmerFinder:
                  only_store_variant_nodes=False,
                  start_at_critical_path_number=None, stop_at_critical_path_number=None,
                  whitelist=None,
-                 only_store_nodes=None):
+                 only_store_nodes=None,
+                 only_follow_nodes=None):
 
         self._graph = graph
         self._linear_ref_nodes = self._graph.linear_ref_nodes()
@@ -46,7 +57,7 @@ class DenseKmerFinder:
         self._kmers = NpList(dtype=np.int64)
         self._allele_frequencies = NpList(dtype=float)
         
-        self._power_vector = np.power(4, np.arange(0, k, dtype=np.uint64))
+        self._power_vector = power_array(k)
 
         self._current_search_start_node = self._graph.get_first_node()
         self._current_search_start_offset = 0
@@ -83,6 +94,7 @@ class DenseKmerFinder:
 
         self._whitelist = whitelist  # set of kmers, do not store any other than these
         self._n_skipped_whitelist = 0
+        self._only_follow_nodes = only_follow_nodes
 
         self._early_stop = False
 
@@ -91,6 +103,9 @@ class DenseKmerFinder:
             self._whitelist = whitelist
             #logging.info("Will limit kmers to whitelist (%d kmers in whitelist)" % len(whitelist))
 
+    def get_found_kmers_and_nodes(self):
+        return (self._kmers.get_nparray(), self._nodes.get_nparray())
+
     def get_flat_kmers(self, v="2"):
         assert self._allele_frequencies.get_nparray().dtype == float
         if v == "0" or v == "1":
@@ -117,11 +132,12 @@ class DenseKmerFinder:
             return
 
         nodes = np.unique(self._current_nodes[self._current_path_start_position:])
-        #logging.info("     Adding kmer %d at node/offset %d/%d with nodes %s. Start node/offset: %d/%d" %
-        #             (kmer, start_node, start_offset, nodes, start_node, start_offset))
+        #print("     Adding kmer %d/%s at node/offset %d/%d with nodes %s. Start node/offset: %d/%d" %
+        #             (kmer, kmer_hash_to_sequence(kmer, self._k), start_node, start_offset, nodes, start_node, start_offset))
 
         n_variant_nodes = len([n for n in nodes if not self._graph.is_linear_ref_node_or_linear_ref_dummy_node(n)])
-        assert n_variant_nodes <= self._max_variant_nodes
+        if n_variant_nodes > self._max_variant_nodes:
+            logging.warning("Passed too many variant nodes")
 
 
         kmer_allele_frequency = np.min(self._graph.get_node_allele_frequencies(nodes))
@@ -262,9 +278,10 @@ class DenseKmerFinder:
             assert first_base != -1
             current_base = self._graph.get_numeric_base_sequence(node, offset)
 
-            only_add = True
+            only_add = self._nonempty_bases_traversed  # len(self._current_bases)
             if current_base != -1:
-                if len(self._current_bases) >= self._k:
+                #if len(self._current_bases) >= self._k:
+                if self._nonempty_bases_traversed >= self._k:
                     self._current_path_start_position += 1
                     only_add = False
                 current_hash = update_hash(current_base, current_hash, first_base, self._k,
@@ -283,18 +300,20 @@ class DenseKmerFinder:
             assert self._nonempty_bases_traversed <= len(self._current_bases)
 
             if False and ((node < 100 and offset < 15) or (offset > 0 and offset % 5000 == 0) or node % 5000 == 0):
-                logging.info("On node %d/%d, offset %d, %d kmers added. Skipped nodes: %d. "
+                print("On node %d/%d, offset %d, %d kmers added. Skipped nodes: %d. "
                              "Path length: %d. Rec depth: %d. Nonempty traversed: %d. Nodes: %s"
                              % (node, len(self._graph.nodes), offset, len(self._kmers),
                                 self._n_nodes_skipped_because_too_complex,
                                 len(self._current_bases), self._recursion_depth, self._nonempty_bases_traversed,
-                                ",".join((str(b) for b in np.unique(self._current_nodes[self._current_path_start_position:])))))
+                                ",".join((str(b) for b in (self._current_nodes[self._current_path_start_position:])))))
                 #logging.info("Current search start node/offset: %d/%d" % (self._current_search_start_node, self._current_search_start_offset))
 
             current_path_desc = (node, offset, frozenset(self._current_nodes[self._current_path_start_position:]))
             if (node != self._current_critical_node or offset != self._current_critical_offset) and \
-                    current_path_desc in self._positions_treated and len(self._current_nodes) >= self._k:
+                    current_path_desc in self._positions_treated and len(self._current_nodes[self._current_path_start_position:]) >= self._k:
                 self._recursion_depth -= 1
+                #logging.info("Stopping recursion")
+                #logging.info("current path desc: %s" % str((current_path_desc)))
                 return False
 
             self._positions_treated.add(current_path_desc)
@@ -363,18 +382,24 @@ class DenseKmerFinder:
 
     def _search_next_nodes(self, current_hash, node):
         next_nodes = self._graph.get_edges(node)
+        force_follow = False
+        if self._only_follow_nodes is not None and len(self._only_follow_nodes.intersection(next_nodes)) > 0:
+            next_nodes = self._only_follow_nodes.intersection(next_nodes)
+            force_follow = True
+
         if len(next_nodes) > 0:
             n_variant_nodes_passed = len(set([n for n in self._current_nodes[self._current_path_start_position:] if
                                               not self._graph.is_linear_ref_node_or_linear_ref_dummy_node(n)]))
             #logging.info("Searching next nodes from %d. Variant nodes until now: %d" % (node, n_variant_nodes_passed))
-            assert n_variant_nodes_passed <= self._max_variant_nodes
+            if n_variant_nodes_passed > self._max_variant_nodes:
+                logging.warning("Passed more variant nodes than planned. Could happen if forcing following certain nodes")
 
-            if n_variant_nodes_passed >= self._max_variant_nodes:
+            if not force_follow and n_variant_nodes_passed >= self._max_variant_nodes:
                 # only allow next nodes on linear ref
                 self._n_nodes_skipped_because_too_complex += len(next_nodes)
                 next_nodes = [node for node in next_nodes if
                               self._graph.is_linear_ref_node_or_linear_ref_dummy_node(node)]
-                assert len(next_nodes) == 1
+                assert len(next_nodes) == 1, "Not 1 linear ref next nodes from node %d: %s" % (node, next_nodes)
                 self._n_nodes_skipped_because_too_complex -= len(next_nodes)
 
             #logging.info("Next nodes: %s" % next_nodes)
