@@ -10,7 +10,8 @@ from .kmer_finder import DenseKmerFinder
 class UniqueVariantKmersFinder:
     def __init__(self, graph, variant_to_nodes, variants, k=31, max_variant_nodes=6,
                  kmer_index_with_frequencies=None, haplotype_matrix=None, node_to_variants=None,
-                 do_not_choose_lowest_frequency_kmers=False, use_dense_kmer_finder=False, position_id_index=None):
+                 do_not_choose_lowest_frequency_kmers=False, use_dense_kmer_finder=False, position_id_index=None,
+                 use_simple=False):
         self.graph = graph
         self.variant_to_nodes = variant_to_nodes
         self.reference_kmer_index = None
@@ -26,6 +27,7 @@ class UniqueVariantKmersFinder:
         self._use_dense_kmer_finder = use_dense_kmer_finder
         self._position_id_index = position_id_index
         self._nodes_found = set()
+        self._use_simple = use_simple
 
         if self._use_dense_kmer_finder:
             assert self._position_id_index is not None, "Position id index must be set when using dense kmer finder"
@@ -45,6 +47,70 @@ class UniqueVariantKmersFinder:
                 return False
         return True
 
+    def _find_all_kmers_from_position_that_covers_node(self, chromosome, ref_position, cover_node):
+        possible_ref_position_adjusted = self.graph.convert_chromosome_ref_offset_to_graph_ref_offset(
+            ref_position, chromosome)
+        finder = DenseKmerFinder(self.graph, self.k, None, position_id=self._position_id_index,
+                                 max_variant_nodes=self._max_variant_nodes, only_store_nodes=set(cover_node),
+                                 )
+        position_node = self.graph.get_node_at_ref_offset(possible_ref_position_adjusted)
+        position_offset = self.graph.get_node_offset_at_ref_offset(possible_ref_position_adjusted)
+        finder.find_only_kmers_starting_at_position(node, offset)
+        return finder.kmers_found()
+
+
+    def _find_all_possible_kmer_sets_covering_node(self):
+        pass
+
+
+    def find_kmers_over_variant_node(self, variant, node):
+        # tries to find the best possible kmers to represent node
+        # if node is small, find only one kmer
+        # if node is large, allow for multiple kmers
+        # tries to choose kmers thare are rare
+
+        start_pos = variant.position
+        assert variant.type is not None, "Variant type must be set"
+        if variant.type != "SNP":
+            start_pos = start_pos + 1  # start pos for indels is 1 base before
+
+        start_pos -= 1  # go from 1-based to 0-based coordinates
+
+        node_sequence = self.graph.get_node_sequence(node)
+        if node_sequence == "" or variant.type != "SNP":
+            # empty node, we cannot start at node, start one node before
+            # do this for both alleles if indel, so that we start from the same pos
+            #logging.info("Finding start node by searching for ref offset %d at chromosomse %d" % (start_pos-1, variant.chromosome))
+            start_node = self.graph.get_node_at_chromosome_and_chromosome_offset(variant.chromosome, start_pos-8)
+            start_node_offset = self.graph.get_node_offset_at_chromosome_and_chromosome_offset(variant.chromosome, start_pos-8)
+            #start_node_offset = self.graph.get_node_size(start_node)-
+        else:
+            start_node = node
+            start_node_offset = 0
+
+        finder = DenseKmerFinder(self.graph, self.k, None, position_id=self._position_id_index,
+                                 max_variant_nodes=self._max_variant_nodes, only_store_nodes=set([node]),
+                                 only_follow_nodes=set([node])
+                                 )
+        #logging.info("Searching from node %d offset %d for variant %s" % (start_node, start_node_offset, variant))
+        finder.find_only_kmers_starting_at_position(start_node, start_node_offset)
+        result = finder.get_flat_kmers(v="1")
+        return result
+
+
+    def find_kmers_over_structural_variant(self, variant, ref_node, variant_node):
+        # find over ref node and variant node
+        self.find_kmers_over_variant_node(variant, ref_node)
+        self.find_kmers_over_variant_node(variant, variant_node)
+
+
+    def find_kmers_over_variant(self, variant, ref_node, variant_node):
+        # more simple version, does not try to find unique kmers
+        ref_kmers = self.find_kmers_over_variant_node(variant, ref_node)
+        variant_kmers = self.find_kmers_over_variant_node(variant, variant_node)
+        return FlatKmers.from_multiple_flat_kmers([ref_kmers, variant_kmers])
+
+
     def find_unique_kmers_over_variant(self, variant, ref_node, variant_node):
         is_valid = False
         has_added = False
@@ -168,7 +234,8 @@ class UniqueVariantKmersFinder:
                                                   (node, variant, ref_node, variant_node)
             self._nodes_found.add(node)
 
-        self.flat_kmers_found.append(best_position)
+        return best_position
+        #self.flat_kmers_found.append(best_position)
 
 
     def find_unique_kmers(self):
@@ -181,13 +248,17 @@ class UniqueVariantKmersFinder:
                 prev_time = time.time()
 
             #ref_node, variant_node = self.graph.get_variant_nodes(variant)
+            assert variant.vcf_line_number is not None, "Variant line number must be specified"
             ref_node = self.variant_to_nodes.ref_nodes[variant.vcf_line_number]
             variant_node = self.variant_to_nodes.var_nodes[variant.vcf_line_number]
 
             if ref_node == 0 or variant_node == 0:
                 continue
 
-            self.find_unique_kmers_over_variant(variant, ref_node, variant_node)
+            if not self._use_simple:
+                self.flat_kmers_found.append(self.find_unique_kmers_over_variant(variant, ref_node, variant_node))
+            else:
+                self.flat_kmers_found.append(self.find_kmers_over_variant(variant, ref_node, variant_node))
 
             if len(self.flat_kmers_found) != n_processed + 1:
                 logging.warning("DID NOT FIND KMERS ON %s" % variant)
